import { minimist, R, constants, log, toGroupedCommands } from './common';
const argv = minimist(process.argv.slice(2));
const maxStringLength = (strings) => Math.max.apply(null, strings.map(item => item.length));
function printCommandHelp(name, command) {
    const args = Object.keys(command.args || {}).map(k => ({
        name: k,
        description: command.args ? command.args[k] : '',
    }));
    const params = args.filter(item => !item.name.startsWith('-'));
    const flags = args.filter(item => item.name.startsWith('-'));
    const paramsDisplay = params.map(p => p.name).join(', ');
    const maxArgLength = maxStringLength(args.map(item => item.name));
    const logArg = (arg, color) => {
        const argName = `${arg.name}${' '.repeat(maxArgLength)}`.substr(0, maxArgLength);
        log.info(`  ${log[color](argName)}  ${arg.description}`);
    };
    log.info();
    log.info(`Usage: ${log.blue(name)} ${log.magenta(paramsDisplay)}`);
    log.info.gray('------------------------------------------------------------');
    log.info(`${command.description || 'No description.'}`);
    log.info();
    if (params.length > 0) {
        log.info.gray('Parameters:');
        params.forEach(arg => logArg(arg, 'magenta'));
        log.info();
    }
    if (flags.length > 0) {
        log.info.gray('Flags:');
        flags.forEach(arg => logArg(arg, 'cyan'));
        log.info();
    }
    log.info();
}
function toDisplayName(command) {
    const { alias, name } = command;
    let displayName = name;
    if (alias.length > 0) {
        displayName = `${displayName} (${alias.join(',')})`;
    }
    return displayName;
}
function printCommands(commands, maxNameLength) {
    const commandNames = Object.keys(commands).sort();
    commandNames.forEach(name => {
        const command = commands[name];
        const { description } = command;
        const displayName = toDisplayName(command);
        const paddedName = `${displayName}${' '.repeat(maxNameLength)}`.substr(0, maxNameLength);
        log.info(`   ${log.blue(paddedName)}  ${log.gray(description || 'No description.')}`);
    });
}
function printGroups(commands) {
    const displayNames = Object.keys(commands).map(k => toDisplayName(commands[k]));
    const maxNameLength = maxStringLength(displayNames);
    const groups = toGroupedCommands(commands);
    Object.keys(groups).forEach(group => {
        if (group !== constants.DEFAULT_GROUP) {
            log.info();
            log.info.gray(` ${group}`);
            log.info();
        }
        printCommands(groups[group], maxNameLength);
    });
}
function findCommand(name, commands) {
    name = name || '';
    name = name.trim();
    if (commands[name]) {
        return commands[name];
    }
    return Object.keys(commands)
        .map(key => commands[key])
        .find(cmd => cmd.alias.includes(name));
}
export default (commands = {}) => {
    const commandName = argv._[0];
    const command = findCommand(commandName, commands);
    const helpRequested = argv.h === true || argv.help === true;
    if (command === undefined) {
        log.info();
        log.info.cyan('Commands:\n');
        printGroups(commands);
        log.info();
        log.info();
        return;
    }
    else if (command && helpRequested) {
        printCommandHelp(commandName, command);
    }
    else {
        const args = argv;
        args._.shift();
        let commandArgs = {
            params: args._,
            options: args,
        };
        delete args._;
        if (command.validate && R.is(Function, command.validate)) {
            commandArgs = command.validate(commandArgs);
        }
        if (commandArgs && R.is(Function, command.action)) {
            try {
                const result = command.action(commandArgs);
                if (result && R.is(Function, result.catch)) {
                    result.catch(err => log.error(err, '\n'));
                }
            }
            catch (err) {
                log.error(err, '\n');
            }
        }
    }
};
